#!/usr/bin/python
# coding:utf-8

# pyweathercn - craw.py
# 2018/5/22 16:31
# get forecast from weather.com.cn

__author__ = 'Benny <benny@bennythink.com>'

import json
import logging
import time
import traceback

import requests
from bs4 import BeautifulSoup

from pyweathercn.constant import CITY, CODE
from pyweathercn.utils import cache


def today_tip(soup):
    s = ''
    tips = soup.find('li', class_='li1')
    s += tips.em.string + tips.span.string + tips.p.string
    tips = soup.find('li', class_='li3 hot')
    s += tips.em.string + tips.span.string + tips.p.string
    return s


def js_hour_aqi(soup):
    script = soup.find_all('script')
    js = script[5].string
    od = json.loads(js[js.index('=') + 2:js.index(';')])
    aqi = [i['od28'] for i in od['od']['od2'] if i['od21'] == time.strftime('%H', time.localtime(3))][0]
    return aqi


def js_hour_temp(soup):
    """
    get hour temperature extract from JavaScript object and dumps it to dict.
    :param soup: beautiful soup object
    :return:temperature
    """
    script = soup.find_all('script')
    js = None
    for item in script:
        if item.string:
            js = item.string
            break
    # js = script[2].string
    od = json.loads(js[js.index('=') + 1:])

    hour = int(time.strftime('%H', time.localtime()))
    if 1 <= hour <= 3:
        hour = '02'
    elif 4 <= hour <= 6:
        hour = '05'
    elif 7 <= hour <= 9:
        hour = '08'
    elif 10 <= hour <= 13:
        hour = '11'
    elif 13 <= hour <= 15:
        hour = '14'
    elif 16 <= hour <= 18:
        hour = '17'
    elif 19 <= hour <= 21:
        hour = '20'
    elif 22 <= hour or hour <= 0:
        hour = '23'

    current_temp = [i.split(',')[3].split('℃')[0] for i in od['1d'] if hour + '时' in i][0]
    return current_temp


def seven_day(soup):
    ul_res = soup.find_all('ul', class_='t clearfix')

    date = wea = tem = win = None
    for ul in ul_res:
        date = ul.find_all('h1')
        wea = ul.find_all('p', class_="wea")
        tem = ul.find_all('p', class_="tem")
        win = ul.find_all('p', class_="win")

    date_list = [data.string for data in date]
    wea_list = [w['title'] for w in wea]
    tem_list = []
    for t in tem:
        if t.span is None:
            tem_list.append(t.i.string)
        else:
            tem_list.append(t.span.string + '/' + t.i.string)

    win_list = [w.em.span['title'] + ' ' + w.i.string for w in win]

    a = [{'date': date_list[i], 'type': wea_list[i], 'temp': tem_list[i], 'wind': win_list[i]} for i in
         range(len(date_list))]
    return a


def convert_city(name):
    return CITY.get(name)


@cache(timeout=10800)
def make_json(city):
    """
    make final request json
    :param city: city name
    :return: result dict
    """
    city_code = convert_city(city)
    # city not found
    if city_code is None:
        return {"code": 400001, "message": CODE[400001], "error": CODE[400001]}

    try:
        url = 'http://www.weather.com.cn/weather/%s.shtml'
        response = requests.get(url % city_code)
        if response.status_code != 200:
            return {"code": 500001, "message": CODE[500001], "error": "Not receiving 200 from weather.com.cn"}

        response.encoding = 'utf-8'
        soup = BeautifulSoup(response.text, 'html.parser')

        sample = {"city": city, "aqi": js_hour_aqi(soup), "tip": today_tip(soup), "temp": js_hour_temp(soup),
                  "forecast": seven_day(soup)}
        response.close()
        return sample
    except Exception:
        logging.error(traceback.format_exc())
        sample = {"code": 500002, "message": CODE[500002], "error": traceback.format_exc()}
        return sample


if __name__ == '__main__':
    print(make_json('深圳'))



#!/usr/bin/python
# coding:utf-8

# pyweathercn - web.py
# 2018/5/22 16:32
# For running up tornado

__author__ = 'Benny <benny@bennythink.com>'

import json
import socket
from platform import uname

from concurrent.futures import ThreadPoolExecutor
from tornado import web, ioloop, httpserver, gen
from tornado.concurrent import run_on_executor
from pyweathercn.utils import api
from pyweathercn.constant import CODE, BANNER, HTTP


class BaseHandler(web.RequestHandler):
    def data_received(self, chunk):
        pass


class IndexHandler(BaseHandler):

    def get(self):
        help_msg = '''Welcome to pyweathercn!
        There are two ways to interact with this RESTAPI.
        The key parameter is city, and an optional parameter day.<br>
        The first one is GET method, invoke:
        <code>127.0.0.1:8888/weather?city=上海</code> - get full details
        <code>127.0.0.1:8888/weather?city=上海&day=2</code> - get 2 days details
        The second one is POST method, invoke <code>127.0.0.1:8888/weather</code>
         with url-encoded form as above. Post JSON is also supported.
        '''.replace('\n', '<br>')

        base = '''<!DOCTYPE html><html><head><title>Welcome to pyweathercn!</title></head>
                <body><pre>{}</pre><br>{}</body></html>'''.format(BANNER, help_msg)
        self.write(base)

    def post(self):
        self.get()


def compatible_get_arguments(self):
    from tornado.web import HTTPError
    try:
        c = self.get_argument('city', None)
    except HTTPError:
        c = self.request.query_arguments['city'][0].decode('gbk')
    return c


class WeatherHandler(BaseHandler):
    executor = ThreadPoolExecutor(max_workers=20)
    PROVIDER = None

    @run_on_executor
    def run_request(self):
        """
        sign and return
        :return: hex and raw request in XML
        """
        # get parameter, compatibility with json
        if WeatherHandler.PROVIDER == 'meizu':
            from pyweathercn.meizu import make_json
        else:
            from pyweathercn.craw import make_json

        if self.request.headers.get('Content-Type') == 'application/json' and self.request.body:
            data = json.loads(self.request.body)
            city = data.get('city')
            day = data.get('day')
        else:
            city = compatible_get_arguments(self)
            day = self.get_argument('day', None)
        # mandatory param missing
        if city is None:
            self.set_status(HTTP.get(400002))
            response = {"code": 400002, "message": CODE[400002], "error": CODE[400002]}
        # day, return specified day.
        elif day:
            data = make_json(city)
            try:
                response = data['forecast'][int(day)]
            except IndexError as e:
                response = {"code": 400003, "message": CODE[400003], "error": str(e)}
            except KeyError as e:
                response = {"code": 400001, "message": CODE[400001], "error": str(e)}

        # return whole json.
        else:
            response = make_json(city)
        # set http code and response
        self.set_status(HTTP.get(response.get('code'), 418))
        return response

    @api
    @gen.coroutine
    def get(self):
        res = yield self.run_request()
        self.write(res)

    @api
    @gen.coroutine
    def post(self):
        res = yield self.run_request()
        self.write(res)


class NotFoundHandler(BaseHandler):
    def prepare(self):  # for all methods
        self.set_status(HTTP[404001])
        response = {"code": 404001, "message": CODE[404001], "error": CODE[404001]}
        self.write(response)

    def get(self):
        pass

    def post(self):
        pass


def get_host_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect(('8.8.8.8', 80))
    ip = s.getsockname()[0]
    s.close()
    return ip


class RunServer:
    handlers = [(r'/weather', WeatherHandler), (r'/', IndexHandler)]
    application = web.Application(handlers, default_handler_class=NotFoundHandler)

    @staticmethod
    def run_server(port=8888, host='0.0.0.0', **kwargs):
        tornado_server = httpserver.HTTPServer(RunServer.application, **kwargs)
        tornado_server.bind(port, host)

        if uname()[0] == 'Windows':
            tornado_server.start()
        else:
            tornado_server.start(None)

        try:
            print(BANNER)
            print('Server is running on http://{}:{}'.format(get_host_ip(), port))
            print('Running server with {}'.format(WeatherHandler.PROVIDER or "weather.com.cn"))
            ioloop.IOLoop.instance().current().start()
        except KeyboardInterrupt:
            ioloop.IOLoop.instance().stop()
            print('"Ctrl+C" received, exiting.\n')


if __name__ == "__main__":
    # import pyweathercn.utils
    # pyweathercn.utils.DB = r'C:\Users\Benny\PycharmProjects\pyweathercn\sample.sqlite'
    WeatherHandler.PROVIDER = 'meizu'
    RunServer.run_server()



#!/usr/bin/python
# coding:utf-8

# pyweathercn - __init__.py
# 2018/5/23 16:32
# 

__author__ = 'Benny <benny@bennythink.com>'

import os

import pyweathercn.web
import pyweathercn.utils

from pyweathercn.craw import make_json


class Weather:
    """
    Weather broadcast class.
    Usage:
    1. chain call:`pyweathercn.Weather('重庆').temp()`
    2. instance mode: ```  w = pyweathercn.Weather('北京')
                            w.data```
    """

    def __init__(self, city_name):
        self.data = make_json(city_name)

    def __del__(self):
        del self.data

    def today(self, raw=False):
        """
        return today weather
        :param raw: return raw json
        :return: json or string.
        """
        return self.__return_result(raw, 0)

    def tomorrow(self, raw=False):
        return self.__return_result(raw, 1)

    def two_days(self, raw=False):
        return self.__return_result(raw, 2)

    def three_days(self, raw=False):
        return self.__return_result(raw, 3)

    def aqi(self, raw=False):
        return self.__make(raw, 'aqi', 'AQI')

    def temp(self, raw=False):
        return self.__make(raw, 'temp', '')

    def tip(self, raw=False):
        return self.__make(raw, 'tip', '温馨提示')

    def forecast(self, raw=False, day=0):
        """
        forecast in assigning days. 0 stands for today, 1 for tomorrow. the number could be no larger than 6.
        :param raw: raw json
        :param day: day option. Default is 0 which will return today's weather information.
        :return: json response
        """
        if raw and self.data.get('code') is None:
            return self.data['forecast'][day]
        elif self.data.get('code') is None:
            return self.data['city'] + '：' + self.__make_str(self.data['forecast'][day])
        else:
            return self.data['message']

    def __make(self, raw, _type, text):
        """
        make specified json
        :param raw: raw json or not
        :param _type: aqi, temp or tip?
        :param text: default tooltip
        :return: json response.
        """
        if raw and self.data.get('code') is None:
            return self.data[_type]

        elif self.data.get('code') is None:
            return self.data['city'] + text + "：" + self.data[_type]
        else:
            return self.data['message']

    def __return_result(self, raw, index):
        return self.forecast(raw, index)

    @staticmethod
    def __make_str(dic):
        s = ''
        for i in dic:
            s = s + dic[i]
        return s


def server(port=8888, host="0.0.0.0", auth=None, provider='meizu', **kwargs):
    """
    run RESTAPI server.
    :param port: the port to listen on. Default is 8888
    :param host: the host to listen on. Default is localhost.
    :param auth: API authentication database path
    :param provider: which provider to use, meizu or weathercn
    :param kwargs: any keyword argument support by tornado. example for SSL:
    ```ssl_options={
        "certfile": "fullchain.pem",
        "keyfile": "privkey.pem"}
    ```
    :return: None
    """
    if auth is not None and not os.path.isfile(auth):
        raise FileNotFoundError('Database file not exists!')
    else:
        pyweathercn.web.WeatherHandler.PROVIDER = provider
        pyweathercn.utils.DB = auth
        pyweathercn.web.RunServer.run_server(port, host, **kwargs)



# coding: utf-8
# pyweathercn - meizu.py
# 2019/2/26 15:37

__author__ = 'Benny <benny@bennythink.com>'

import requests

from pyweathercn.utils import cache
from pyweathercn.constant import CITY, CODE


@cache(timeout=10800)
def make_json(city):
    city_code = CITY.get(city)
    # city not found
    if city_code is None:
        return {"code": 400001, "message": CODE[400001], "error": CODE[400001]}

    url = 'http://aider.meizu.com/app/weather/listWeather?cityIds={}'.format(city_code)
    sample = requests.get(url).json()
    tip, forecast, result = '', [], {}

    for each in sample['value'][0]['indexes']:
        tip = tip + '{}：{}\n'.format(each["name"], each["content"])

    aqi = "{} {}".format(sample['value'][0]['pm25']['aqi'], sample['value'][0]['pm25']['quality'])

    temp = sample['value'][0]['realtime']['temp']

    for item in sample['value'][0]['weathers']:
        date = "{} {}".format(item['date'], item['week'])
        _type = item['weather']
        temp = '{}/{}℃'.format(item["temp_day_c"], item["temp_night_c"])
        sun_rise = item["sun_rise_time"]
        sun_down = item["sun_down_time"]
        forecast.append(dict(date=date, type=_type, temp=temp, sun_rise=sun_rise, sun_down=sun_down, wind=''))

    result['city'] = sample['value'][0]['city']
    result['aqi'] = aqi
    result['tip'] = tip
    result['temp'] = temp
    result['forecast'] = forecast
    result['wind'] = "{} {}".format(sample['value'][0]['realtime']['wD'], sample['value'][0]['realtime']['wS'])

    return result


if __name__ == '__main__':
    r = make_json('南京')
    print(r)



#!/usr/bin/python
# coding: utf-8

# pyweathercn - utils.py
# 2018/8/18 15:12
# decorator utilities, for API Key authentication and so on

__author__ = "Benny <benny@bennythink.com>"

import logging
import json
import socket
import sqlite3

import redis

from pyweathercn.constant import CODE

DB = ":memory:"
logging.basicConfig(level=logging.INFO)


class RequireApi:
    def __init__(self, k):
        if DB == ":memory:" or DB is None:
            # disable auth
            self.__auth = False
            self.__msg = None
            self.__key = None
        elif k is None:
            # missed key
            self.con = sqlite3.connect(DB)
            self.cur = self.con.cursor()
            self.__auth = True
            self.__msg = {"status": 6, "message": CODE.get(6)}
            self.__key = None
        else:
            self.con = sqlite3.connect(DB)
            self.cur = self.con.cursor()
            self.__auth = True
            self.__msg = None
            self.__key = k

    def __del__(self):
        if self.__auth:
            self.con.close()

    def auth(self):
        # return True(contine) or False
        if not self.__auth:
            v = (True, None)
        else:
            self.cur.execute('SELECT times,restrict FROM auth WHERE key=?', (self.__key,))
            data = self.cur.fetchall()

            if data and data[0][1] == 0:
                # unlimited user
                v = (True, None)
            elif data and data[0][0] >= 1:
                # valid, limited user. we need to decrease.
                self.cur.execute('UPDATE auth SET times=? WHERE key=?', (data[0][0] - 1, self.__key))
                self.con.commit()
                v = (True, None)
            elif data and data[0][0] <= 0:
                # exceed
                v = (False, {"code": 429001, "message": CODE[429001], "error": CODE[429001]})
            else:
                # invalid key provided
                v = (False, {"code": 401001, "message": CODE[401001], "error": CODE[401001]})

        return v

    @staticmethod
    def get_key(s):
        # get parameter, compatibility with json
        if s.request.headers.get('Content-Type') == 'application/json' and s.request.body:
            data = json.loads(s.request.body)
            key = data.get('key')
        else:
            key = s.get_argument('key', None)

        return key


def api(fun):
    def wrapper(self):
        key = RequireApi.get_key(self)
        k, msg = RequireApi(key).auth()
        if k:
            res = fun(self)
            return res
        else:
            self.write(msg)

    return wrapper


class TestCache:
    def __init__(self, timeout):
        self.__timeout = timeout
        self.r = redis.Redis(decode_responses=True)

    def retrieve(self, city):
        result = self.r.get(city)
        if result:
            return json.loads(self.r.get(city))

    def update(self, city, weather):
        if weather.get('code') is None:
            self.r.set(city, json.dumps(weather, ensure_ascii=False), ex=self.__timeout)

    @staticmethod
    def check_redis():
        try:
            s = socket.socket()
            s.connect(("127.0.0.1", 6379))
            s.close()
            return True
        except ConnectionRefusedError:
            return False


def cache(timeout):
    def func(fun):
        def inner(*args):
            if TestCache.check_redis():
                t = TestCache(timeout)
                valid = t.retrieve(args[0])
                if valid:
                    logging.info('Retrieving data from redis')
                    return valid
                else:
                    logging.info('Cache expired. Re-requesting now...')
                    res = fun(args[0])
                    t.update(args[0], res)
                    return res
            else:
                logging.warning('Please install Redis for better performance!')
                res = fun(args[0])
                return res

        return inner

    return func
